@@ -18,7 +18,9 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-
+#ifdef _WIN32
+#include "sb_win.h"
+#endif
 #include "db_driver.h"
 #include "sb_list.h"
 
