@@ -21,6 +21,9 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
+#ifdef _WIN32
+#include <winsock2.h>
+#endif
 
 #ifdef HAVE_STRING_H
 # include <string.h>
@@ -122,14 +125,15 @@ db_mysql_bind_map_t db_mysql_bind_map[] =
 
 static drv_caps_t mysql_drv_caps =
 {
-  .multi_rows_insert = 1,
-  .transactions = 0,
-  .prepared_statements = 0,
-  .auto_increment = 1,
-  .serial = 0,
-  .unsigned_int = 1,
+  1,
+  0,
+  0,
+  1,
+  0,
+  0,
+  1,
   
-  .table_options_str = NULL
+  NULL
 };
 
 
@@ -167,10 +171,9 @@ static int mysql_drv_done(void);
 
 static db_driver_t mysql_driver =
 {
-  .sname = "mysql",
-  .lname = "MySQL driver",
-  .args = mysql_drv_args,
-  .ops =
+  "mysql",
+  "MySQL driver",
+  mysql_drv_args,
   {
     mysql_drv_init,
     mysql_drv_describe,
@@ -189,7 +192,7 @@ static db_driver_t mysql_driver =
     mysql_drv_store_results,
     mysql_drv_done
   },
-  .listitem = {NULL, NULL}
+  {NULL, NULL}
 };
 
 
@@ -265,6 +268,8 @@ int mysql_drv_init(void)
   
   s = sb_get_value_string("mysql-table-engine");
 
+  mysql_library_init(0, NULL, NULL);
+  
   return parse_table_engine(s);
 }
 
@@ -477,8 +482,8 @@ int mysql_drv_prepare(db_stmt_t *stmt, const char *query)
       DEBUG("mysql_errno(%p) = %u", con, rc);
       if (rc == ER_UNSUPPORTED_PS)
       {
-        log_text(LOG_WARNING,
-                 "Preparing of \"%s\" is unsupported, using emulation",
+        log_text(LOG_DEBUG,
+                 "Failed to prepare query \"%s\", using emulation",
                  query);
         goto emulate;
       }
@@ -900,6 +905,8 @@ int mysql_drv_close(db_stmt_t *stmt)
 /* Uninitialize driver */
 int mysql_drv_done(void)
 {
+  mysql_library_end();
+  
   return 0;
 }
 
